@@ -1184,6 +1184,46 @@
         return Stateful;
     }
 
+    const MessageType = {
+        UI_GET_DATA: "ui-get-data",
+        UI_GET_ACTIVE_TAB_INFO: "ui-get-active-tab-info",
+        UI_SUBSCRIBE_TO_CHANGES: "ui-subscribe-to-changes",
+        UI_UNSUBSCRIBE_FROM_CHANGES: "ui-unsubscribe-from-changes",
+        UI_CHANGE_SETTINGS: "ui-change-settings",
+        UI_SET_THEME: "ui-set-theme",
+        UI_SET_SHORTCUT: "ui-set-shortcut",
+        UI_TOGGLE_URL: "ui-toggle-url",
+        UI_MARK_NEWS_AS_READ: "ui-mark-news-as-read",
+        UI_LOAD_CONFIG: "ui-load-config",
+        UI_APPLY_DEV_DYNAMIC_THEME_FIXES: "ui-apply-dev-dynamic-theme-fixes",
+        UI_RESET_DEV_DYNAMIC_THEME_FIXES: "ui-reset-dev-dynamic-theme-fixes",
+        UI_APPLY_DEV_INVERSION_FIXES: "ui-apply-dev-inversion-fixes",
+        UI_RESET_DEV_INVERSION_FIXES: "ui-reset-dev-inversion-fixes",
+        UI_APPLY_DEV_STATIC_THEMES: "ui-apply-dev-static-themes",
+        UI_RESET_DEV_STATIC_THEMES: "ui-reset-dev-static-themes",
+        UI_SAVE_FILE: "ui-save-file",
+        UI_REQUEST_EXPORT_CSS: "ui-request-export-css",
+        BG_CHANGES: "bg-changes",
+        BG_ADD_CSS_FILTER: "bg-add-css-filter",
+        BG_ADD_STATIC_THEME: "bg-add-static-theme",
+        BG_ADD_SVG_FILTER: "bg-add-svg-filter",
+        BG_ADD_DYNAMIC_THEME: "bg-add-dynamic-theme",
+        BG_EXPORT_CSS: "bg-export-css",
+        BG_UNSUPPORTED_SENDER: "bg-unsupported-sender",
+        BG_CLEAN_UP: "bg-clean-up",
+        BG_RELOAD: "bg-reload",
+        BG_FETCH_RESPONSE: "bg-fetch-response",
+        BG_UI_UPDATE: "bg-ui-update",
+        BG_CSS_UPDATE: "bg-css-update",
+        CS_COLOR_SCHEME_CHANGE: "cs-color-scheme-change",
+        CS_FRAME_CONNECT: "cs-frame-connect",
+        CS_FRAME_FORGET: "cs-frame-forget",
+        CS_FRAME_FREEZE: "cs-frame-freeze",
+        CS_FRAME_RESUME: "cs-frame-resume",
+        CS_EXPORT_CSS_RESPONSE: "cs-export-css-response",
+        CS_FETCH: "cs-fetch"
+    };
+
     const userAgent =
         typeof navigator === "undefined"
             ? "some useragent"
@@ -1219,6 +1259,10 @@
             return false;
         }
     })();
+    globalThis.chrome &&
+        globalThis.chrome.runtime &&
+        globalThis.chrome.runtime.getManifest &&
+        globalThis.chrome.runtime.getManifest().manifest_version === 3;
 
     function classes(...args) {
         const classes = [];
@@ -1427,8 +1471,31 @@
         }
         throw new Error(`Unable to parse ${$color}`);
     }
-    function getNumbersFromString(str, splitter, range, units) {
-        const raw = str.split(splitter).filter((x) => x);
+    function getNumbers($color) {
+        const numbers = [];
+        let prevPos = 0;
+        let isMining = false;
+        const startIndex = $color.indexOf("(");
+        $color = $color.substring(startIndex + 1, $color.length - 1);
+        for (let i = 0; i < $color.length; i++) {
+            const c = $color[i];
+            if ((c >= "0" && c <= "9") || c === "." || c === "+" || c === "-") {
+                isMining = true;
+            } else if (isMining && (c === " " || c === ",")) {
+                numbers.push($color.substring(prevPos, i));
+                isMining = false;
+                prevPos = i + 1;
+            } else if (!isMining) {
+                prevPos = i + 1;
+            }
+        }
+        if (isMining) {
+            numbers.push($color.substring(prevPos, $color.length));
+        }
+        return numbers;
+    }
+    function getNumbersFromString(str, range, units) {
+        const raw = getNumbers(str);
         const unitsList = Object.entries(units);
         const numbers = raw
             .map((r) => r.trim())
@@ -1450,28 +1517,16 @@
             });
         return numbers;
     }
-    const rgbSplitter = /rgba?|\(|\)|\/|,|\s/gi;
     const rgbRange = [255, 255, 255, 1];
     const rgbUnits = {"%": 100};
     function parseRGB($rgb) {
-        const [r, g, b, a = 1] = getNumbersFromString(
-            $rgb,
-            rgbSplitter,
-            rgbRange,
-            rgbUnits
-        );
+        const [r, g, b, a = 1] = getNumbersFromString($rgb, rgbRange, rgbUnits);
         return {r, g, b, a};
     }
-    const hslSplitter = /hsla?|\(|\)|\/|,|\s/gi;
     const hslRange = [360, 1, 1, 1];
     const hslUnits = {"%": 100, "deg": 360, "rad": 2 * Math.PI, "turn": 1};
     function parseHSL($hsl) {
-        const [h, s, l, a = 1] = getNumbersFromString(
-            $hsl,
-            hslSplitter,
-            hslRange,
-            hslUnits
-        );
+        const [h, s, l, a = 1] = getNumbersFromString($hsl, hslRange, hslUnits);
         return hslToRGB({h, s, l, a});
     }
     function parseHex($hex) {
@@ -2108,10 +2163,51 @@
         });
         return context.leave();
     }
-    Object.assign(Overlay, {Portal});
+    var Overlay$1 = Object.assign(Overlay, {Portal});
+
+    function MessageBox(props) {
+        return m(
+            Overlay$1.Portal,
+            {key: props.portalKey, onOuterClick: props.onCancel},
+            m(
+                "div",
+                {class: "message-box"},
+                m("label", {class: "message-box__caption"}, props.caption),
+                m(
+                    "div",
+                    {class: "message-box__buttons"},
+                    m(
+                        Button,
+                        {
+                            class: "message-box__button message-box__button-ok",
+                            onclick: props.onOK
+                        },
+                        "OK"
+                    ),
+                    m(
+                        Button,
+                        {
+                            class: "message-box__button message-box__button-cancel",
+                            onclick: props.onCancel
+                        },
+                        "Cancel"
+                    )
+                )
+            )
+        );
+    }
 
     function getUILanguage() {
-        const code = chrome.i18n.getUILanguage();
+        let code;
+        if (
+            "i18n" in chrome &&
+            "getUILanguage" in chrome.i18n &&
+            typeof chrome.i18n.getUILanguage === "function"
+        ) {
+            code = chrome.i18n.getUILanguage();
+        } else {
+            code = navigator.language.split("-")[0];
+        }
         if (code.endsWith("-mac")) {
             return code.substring(0, code.length - 4);
         }
@@ -2128,7 +2224,7 @@
     };
 
     const DEVTOOLS_DOCS_URL =
-        "https://github.com/darkreader/darkreader#how-to-contribute";
+        "https://github.com/darkreader/darkreader/blob/master/CONTRIBUTING.md";
 
     function isIPV6(url) {
         const openingBracketIndex = url.indexOf("[");
@@ -2250,6 +2346,7 @@
     }
 
     function Body({data, tab, actions}) {
+        const context = getComponentContext();
         const {state, setState} = useState({errorText: null});
         let textNode;
         const previewButtonText = data.settings.previewNewDesign
@@ -2313,7 +2410,25 @@
                 });
             }
         }
+        function showDialog() {
+            context.store.isDialogVisible = true;
+            context.refresh();
+        }
+        function hideDialog() {
+            context.store.isDialogVisible = false;
+            context.refresh();
+        }
+        const dialog =
+            context && context.store.isDialogVisible
+                ? m(MessageBox, {
+                      caption:
+                          "Are you sure you want to remove current changes? You cannot restore them later.",
+                      onOK: reset,
+                      onCancel: hideDialog
+                  })
+                : null;
         function reset() {
+            context.store.isDialogVisible = false;
             wrapper.reset();
             setState({errorText: null});
         }
@@ -2348,7 +2463,7 @@
             m(
                 "div",
                 {id: "buttons"},
-                m(Button, {onclick: reset}, "Reset"),
+                m(Button, {onclick: showDialog}, "Reset changes", dialog),
                 m(Button, {onclick: apply}, "Apply"),
                 m(
                     Button,
@@ -2377,112 +2492,169 @@
                 m("strong", null, "popular"),
                 " website looks incorrect e-mail to ",
                 m("strong", null, "DarkReaderApp@gmail.com")
-            )
+            ),
+            m(Overlay$1, null)
         );
     }
     var Body$1 = withState(Body);
 
     class Connector {
         constructor() {
-            this.counter = 0;
-            this.port = chrome.runtime.connect({name: "ui"});
+            this.onChangesReceived = ({type, data}) => {
+                if (type === MessageType.BG_CHANGES) {
+                    this.changeSubscribers.forEach((callback) =>
+                        callback(data)
+                    );
+                }
+            };
+            this.changeSubscribers = new Set();
         }
-        getRequestId() {
-            return ++this.counter;
-        }
-        async sendRequest(request, executor) {
-            const id = this.getRequestId();
+        async sendRequest(type, data) {
             return new Promise((resolve, reject) => {
-                const listener = ({id: responseId, ...response}) => {
-                    if (responseId === id) {
-                        executor(response, resolve, reject);
-                        this.port.onMessage.removeListener(listener);
+                chrome.runtime.sendMessage({type, ...data}, ({data, error}) => {
+                    if (error) {
+                        reject(error);
+                    } else {
+                        resolve(data);
                     }
-                };
-                this.port.onMessage.addListener(listener);
-                this.port.postMessage({...request, id});
+                });
+            });
+        }
+        async firefoxSendRequestWithResponse(type, data) {
+            return new Promise((resolve, reject) => {
+                const dataPort = chrome.runtime.connect({name: type});
+                dataPort.onDisconnect.addListener(() => reject());
+                dataPort.onMessage.addListener(({data, error}) => {
+                    if (error) {
+                        reject(error);
+                    } else {
+                        resolve(data);
+                    }
+                    dataPort.disconnect();
+                });
+                data && dataPort.postMessage(data);
             });
         }
         async getData() {
-            return await this.sendRequest(
-                {type: "get-data"},
-                ({data}, resolve) => resolve(data)
-            );
+            if (isFirefox) {
+                return await this.firefoxSendRequestWithResponse(
+                    MessageType.UI_GET_DATA
+                );
+            }
+            return await this.sendRequest(MessageType.UI_GET_DATA);
         }
         async getActiveTabInfo() {
-            return await this.sendRequest(
-                {type: "get-active-tab-info"},
-                ({data}, resolve) => resolve(data)
-            );
+            if (isFirefox) {
+                return await this.firefoxSendRequestWithResponse(
+                    MessageType.UI_GET_ACTIVE_TAB_INFO
+                );
+            }
+            return await this.sendRequest(MessageType.UI_GET_ACTIVE_TAB_INFO);
         }
         subscribeToChanges(callback) {
-            const id = this.getRequestId();
-            this.port.onMessage.addListener(({id: responseId, data}) => {
-                if (responseId === id) {
-                    callback(data);
-                }
-            });
-            this.port.postMessage({type: "subscribe-to-changes", id});
-        }
-        enable() {
-            this.port.postMessage({type: "enable"});
-        }
-        disable() {
-            this.port.postMessage({type: "disable"});
+            this.changeSubscribers.add(callback);
+            if (this.changeSubscribers.size === 1) {
+                chrome.runtime.onMessage.addListener(this.onChangesReceived);
+                chrome.runtime.sendMessage({
+                    type: MessageType.UI_SUBSCRIBE_TO_CHANGES
+                });
+            }
         }
         setShortcut(command, shortcut) {
-            this.port.postMessage({
-                type: "set-shortcut",
+            chrome.runtime.sendMessage({
+                type: MessageType.UI_SET_SHORTCUT,
                 data: {command, shortcut}
             });
         }
         changeSettings(settings) {
-            this.port.postMessage({type: "change-settings", data: settings});
+            chrome.runtime.sendMessage({
+                type: MessageType.UI_CHANGE_SETTINGS,
+                data: settings
+            });
         }
         setTheme(theme) {
-            this.port.postMessage({type: "set-theme", data: theme});
+            chrome.runtime.sendMessage({
+                type: MessageType.UI_SET_THEME,
+                data: theme
+            });
         }
         toggleURL(url) {
-            this.port.postMessage({type: "toggle-url", data: url});
+            chrome.runtime.sendMessage({
+                type: MessageType.UI_TOGGLE_URL,
+                data: url
+            });
         }
         markNewsAsRead(ids) {
-            this.port.postMessage({type: "mark-news-as-read", data: ids});
+            chrome.runtime.sendMessage({
+                type: MessageType.UI_MARK_NEWS_AS_READ,
+                data: ids
+            });
         }
         loadConfig(options) {
-            this.port.postMessage({type: "load-config", data: options});
+            chrome.runtime.sendMessage({
+                type: MessageType.UI_LOAD_CONFIG,
+                data: options
+            });
         }
         async applyDevDynamicThemeFixes(text) {
+            if (isFirefox) {
+                return await this.firefoxSendRequestWithResponse(
+                    MessageType.UI_APPLY_DEV_DYNAMIC_THEME_FIXES,
+                    {data: text}
+                );
+            }
             return await this.sendRequest(
-                {type: "apply-dev-dynamic-theme-fixes", data: text},
-                ({error}, resolve, reject) =>
-                    error ? reject(error) : resolve()
+                MessageType.UI_APPLY_DEV_DYNAMIC_THEME_FIXES,
+                {data: text}
             );
         }
         resetDevDynamicThemeFixes() {
-            this.port.postMessage({type: "reset-dev-dynamic-theme-fixes"});
+            chrome.runtime.sendMessage({
+                type: MessageType.UI_RESET_DEV_DYNAMIC_THEME_FIXES
+            });
         }
         async applyDevInversionFixes(text) {
+            if (isFirefox) {
+                return await this.firefoxSendRequestWithResponse(
+                    MessageType.UI_APPLY_DEV_INVERSION_FIXES,
+                    {data: text}
+                );
+            }
             return await this.sendRequest(
-                {type: "apply-dev-inversion-fixes", data: text},
-                ({error}, resolve, reject) =>
-                    error ? reject(error) : resolve()
+                MessageType.UI_APPLY_DEV_INVERSION_FIXES,
+                {data: text}
             );
         }
         resetDevInversionFixes() {
-            this.port.postMessage({type: "reset-dev-inversion-fixes"});
+            chrome.runtime.sendMessage({
+                type: MessageType.UI_RESET_DEV_INVERSION_FIXES
+            });
         }
         async applyDevStaticThemes(text) {
+            if (isFirefox) {
+                return await this.firefoxSendRequestWithResponse(
+                    MessageType.UI_APPLY_DEV_STATIC_THEMES,
+                    {data: text}
+                );
+            }
             return await this.sendRequest(
-                {type: "apply-dev-static-themes", data: text},
-                ({error}, resolve, reject) =>
-                    error ? reject(error) : resolve()
+                MessageType.UI_APPLY_DEV_STATIC_THEMES,
+                {data: text}
             );
         }
         resetDevStaticThemes() {
-            this.port.postMessage({type: "reset-dev-static-themes"});
+            chrome.runtime.sendMessage({
+                type: MessageType.UI_RESET_DEV_STATIC_THEMES
+            });
         }
         disconnect() {
-            this.port.disconnect();
+            if (this.changeSubscribers.size > 0) {
+                this.changeSubscribers.clear();
+                chrome.runtime.onMessage.removeListener(this.onChangesReceived);
+                chrome.runtime.sendMessage({
+                    type: MessageType.UI_UNSUBSCRIBE_FROM_CHANGES
+                });
+            }
         }
     }
 
@@ -2493,6 +2665,7 @@
                 isReady: true,
                 settings: {
                     enabled: true,
+                    fetchNews: true,
                     presets: [],
                     theme: {
                         mode: 1,
@@ -2515,7 +2688,6 @@
                     changeBrowserTheme: false,
                     enableForPDF: true,
                     enableForProtectedPages: false,
-                    notifyOfNews: false,
                     syncSettings: true,
                     automation: "",
                     time: {
@@ -2556,7 +2728,8 @@
         return {
             url: "https://darkreader.org/",
             isProtected: false,
-            isInDarkList: false
+            isInDarkList: false,
+            isInjected: true
         };
     }
     function createConnectorMock() {
